from vcCommand import *

app = getApplication()
selection_manager = app.SelectionManager

def OnStart():
  selected_statements = selection_manager.getSelection(VC_SELECTION_STATEMENT)
  if len(selected_statements) > 0:
    selected_statement = selected_statements[0]
    routine = selected_statement.ParentRoutine
    component = routine.Program.Executor.Component   
    
    all_lines_signal = component.findBehaviour("AllLinesSignal")
    if not all_lines_signal:
      all_lines_signal = component.createBehaviour(VC_BOOLEANSIGNAL, "AllLinesSignal")
    
    specific_index_signal = component.findBehaviour("SpecificIndexSignal")    
    if not specific_index_signal:
      specific_index_signal = component.createBehaviour(VC_INTEGERSIGNAL, "SpecificIndexSignal")
    
    specific_variable_name_signal = component.findBehaviour("SpecificVariableNameSignal")
    if not specific_variable_name_signal:
      specific_variable_name_signal = component.createBehaviour(VC_STRINGSIGNAL, "SpecificVariableNameSignal")
   
    specific_variable_value_signal = component.findBehaviour("SpecificVariableValueSignal")
    if not specific_variable_value_signal:
      specific_variable_value_signal = component.createBehaviour(VC_STRINGSIGNAL, "SpecificVariableValueSignal")
    
    note = component.findBehaviour("Note")
    if not note:
      note = component.createBehaviour(VC_NOTE, "Note")       
    
    handler = component.findBehaviour("VariablesCache")
    if handler:
      handler.delete()
    handler = component.createBehaviour(VC_PYTHONPROCESSHANDLER, "VariablesCache")
    
    all_lines_signal.Connections = [handler]
    specific_index_signal.Connections = [handler]
    specific_variable_name_signal.Connections = [handler]
    specific_variable_value_signal.Connections = [handler]
        
    handler.Script = '''from vcPythonProcessHandler import *
import xlrd, functools, sys

app = getApplication()
comp = getComponent()
this = comp.findBehaviour(__file__.split('::')[1])
transport_node = comp.findBehavioursByType(VC_TRANSPORTNODE)[0]
process_executor = transport_node.ProcessExecutor
all_lines_signal = comp.findBehaviour("AllLinesSignal")
specific_index_signal = comp.findBehaviour("SpecificIndexSignal")
specific_variable_name_signal = comp.findBehaviour("SpecificVariableNameSignal")
specific_variable_value_signal = comp.findBehaviour("SpecificVariableValueSignal")
note = comp.findBehaviour("Note")

def get_possible_type(value):    
  try:
    float_value = float(value)
    if value.endswith('.0'):
      return VC_INTEGER
    else:
      return VC_REAL
  except ValueError:
    pass
    
  try:
    int_value = int(value)        
    return VC_INTEGER
  except ValueError:
    pass
    
  if value.lower() in ['true', 'false']:
    return VC_BOOLEAN
    
  return VC_STRING

def OnSelectExcel(property, statement):    
  sheet_name = statement.getProperty("SheetName")
  sheet_name.StepValues = []
  if statement.Excel[8:] != "":
    book =  xlrd.open_workbook(statement.Excel[8:]) 
    sheet_name.StepValues = [str(_sheet_name) for _sheet_name in book._sheet_names]
    sheet_name.Value = sheet_name.StepValues[0]
    book.release_resources()

def OnSelectSheet(property, statement):
  if statement.Excel[8:] != "":
    book =  xlrd.open_workbook(statement.Excel[8:]) 
    sheet = book.sheet_by_name(statement.SheetName)
    statement.FromRow = 1
    statement.ToRow = sheet.nrows
    book.release_resources()
    
def OnReadExcel(property, statement):    
  note.Note = ""
  
  book =  xlrd.open_workbook(statement.Excel[8:])  
  sheet = book.sheet_by_name(statement.SheetName)
  rows = sheet.nrows
  cols = sheet.ncols
  book.release_resources()
  # OnSelectSheet("", statement)
  
  all_possible_types = [[] for col_index in range(cols)]
  property_names = []
  lines = []
  for row_index in range(statement.FromRow - 1, statement.ToRow):    
    row_cells = []    
    
    for col_index in range(cols):    
      cell_value = str(sheet.cell_value(row_index, col_index))
      cell_value = cell_value.replace("\\n", " ")
      row_cells.append(cell_value)
      all_possible_types[col_index].append(get_possible_type(cell_value))
    
    line = ", ".join(row_cells)
    lines.append(line)    
    
    if row_index == statement.FromRow - 1:
      property_names = row_cells
      
  note.Note = "\\n".join(lines)    
  note.Note = note.Note.replace('\\r\\n','\\n')
  
  print "%s %s: %s %s" % ("-" * 30, routine.Program.Executor.Component.Name, routine.Name, "-" * 30)
  for possible_types_index, possible_types in enumerate(all_possible_types): 
    most_possible_type = max(set(possible_types), key=possible_types.count)
    property_name = property_names[possible_types_index]
    property = routine.getProperty(property_name)
    if not property:
      routine.createProperty(most_possible_type, property_name)
    
    print "Variable '%s' type: '%s'" % (property_name, most_possible_type)
  
  OnUpdateVariableOptions("", statement)
  
def OnUpdateVariableOptions(property, statement):
  property_names = [p.Name for p in routine.Properties if p.Name not in ["Name", "Description"]]  
  property_names.sort()
  property_names.insert(0, "")
    
  p = statement.getProperty('NextTransportPatternIn::Offset.Tx')
  p.StepValues = property_names
  p = statement.getProperty('NextTransportPatternIn::Offset.Ty')
  p.StepValues = property_names
  p = statement.getProperty('NextTransportPatternIn::Offset.Tz')
  p.StepValues = property_names
  p = statement.getProperty('NextTransportPatternIn::Offset.Rx')
  p.StepValues = property_names
  p = statement.getProperty('NextTransportPatternIn::Offset.Ry')
  p.StepValues = property_names
  p = statement.getProperty('NextTransportPatternIn::Offset.Rz')
  p.StepValues = property_names
  p = statement.getProperty('NextTransportPatternIn::PatternCount.X')
  p.StepValues = property_names
  p = statement.getProperty('NextTransportPatternIn::PatternCount.Y')
  p.StepValues = property_names
  p = statement.getProperty('NextTransportPatternIn::PatternCount.Z')
  p.StepValues = property_names
  p = statement.getProperty('NextTransportPatternIn::PatternStep.X')
  p.StepValues = property_names
  p = statement.getProperty('NextTransportPatternIn::PatternStep.Y')
  p.StepValues = property_names
  p = statement.getProperty('NextTransportPatternIn::PatternStep.Z')
  p.StepValues = property_names  
  p = statement.getProperty('NextTransportPatternIn::FilterProductPropertyValue')
  p.StepValues = property_names
  
def get_routine(arg):
  global routine
  if len(process_executor.Processes) > 0:
    routine = process_executor.Processes[0]    
    statements = [statement for statement in routine.Statements if statement.Type == VC_STATEMENT_PROCESS and statement.Process == this]
    if len(statements):
      statement = statements[0]
      hide_gui(statement)
      set_gui(statement)  
      set_gui_events(statement)
      app.OnRender = None
      
      global excel, sheet_name, read_excel, update_variable_options  
      excel = statement.getProperty("Excel")      
      excel.OnChanged = lambda arg: OnSelectExcel(arg, statement)
      sheet_name = statement.getProperty("SheetName")
      sheet_name.OnChanged = lambda arg: OnSelectSheet(arg, statement)
      read_excel = statement.getProperty("ReadExcel")
      read_excel.OnChanged = lambda arg: OnReadExcel(arg, statement)
      update_variable_options = statement.getProperty("NextTransportPatternIn::Update Variable Options")
      update_variable_options.OnChanged = lambda arg: OnUpdateVariableOptions(arg, statement)      
      
app.OnRender = get_routine

def gui_events(arg):
  if gui_execution_mode.Value == "Statement":
    gui_loop_time.IsVisible = True
    gui_infinite_loop.IsVisible = True
    
    if gui_reading_mode.Value == "All lines":
      gui_specific_index.IsVisible = False
      gui_specific_variable_name.IsVisible = False
      gui_specific_variable_value.IsVisible = False
    elif gui_reading_mode.Value == "Specific index":
      gui_specific_index.IsVisible = True
      gui_specific_variable_name.IsVisible = False
      gui_specific_variable_value.IsVisible = False
    elif gui_reading_mode.Value == "Specific variable name and value":
      gui_specific_index.IsVisible = False
      gui_specific_variable_name.IsVisible = True
      gui_specific_variable_value.IsVisible = True

  elif gui_execution_mode.Value == "Signal":
    gui_loop_time.IsVisible = False
    gui_infinite_loop.IsVisible = False

    gui_specific_index.IsVisible = False
    gui_specific_variable_name.IsVisible = False
    gui_specific_variable_value.IsVisible = False

def set_gui_events(statement):
  global gui_execution_mode, gui_loop_time, gui_infinite_loop
  gui_execution_mode = statement.getProperty('ExecutionMode')  
  gui_execution_mode.OnChanged = gui_events
  gui_loop_time = statement.getProperty('LoopTime')  
  gui_infinite_loop = statement.getProperty('InfiniteLoop')  
  
  global gui_reading_mode, gui_specific_index, gui_specific_variable_name, gui_specific_variable_value
  gui_reading_mode = statement.getProperty('ReadingMode')  
  gui_reading_mode.OnChanged = gui_events
  gui_specific_index = statement.getProperty('SpecificIndex')  
  gui_specific_variable_name = statement.getProperty('SpecificVariableName')  
  gui_specific_variable_value = statement.getProperty('SpecificVariableValue') 

  gui_events("")   
  
def hide_gui(statement):  
  statement.getProperty('Name').IsVisible = False
  statement.getProperty('icon').IsVisible = False
  statement.getProperty('Tool').IsVisible = False
  statement.getProperty('Base').IsVisible = False

def unwritable(property, only_when_simulatiing = False):
  property.WritableWhenSimulating = False
  if not only_when_simulatiing:
    property.WritableWhenConnected = False
    property.WritableWhenDisconnected = False  

def set_gui(statement):
  process = statement.getProperty("Process")  
  unwritable(process)
  excel = statement.getProperty("Excel")  
  if not excel:
    excel = statement.createProperty(VC_URI, "Excel")
    excel.ChangeOnRebuild = False    
    excel.IsVisible = True       
    excel.Group = 90
  unwritable(excel, True)  
  sheet_name = statement.getProperty("SheetName")  
  unwritable(sheet_name, True)  
  from_row = statement.getProperty("FromRow")  
  unwritable(from_row, True)  
  to_row = statement.getProperty("ToRow")  
  unwritable(to_row, True)  
  read_excel = statement.getProperty("ReadExcel")  
  unwritable(read_excel, True)  
  
  current_loop = statement.getProperty("CurrentLoop")
  current_step = statement.getProperty("CurrentStep")
  variables = statement.getProperty("Variables")  
  unwritable(current_loop)  
  unwritable(current_step)  
  unwritable(variables)

  loop_time = statement.getProperty("LoopTime")
  infinite_loop = statement.getProperty("InfiniteLoop")
  reading_mode = statement.getProperty("ReadingMode")
  specific_indexes = statement.getProperty("SpecificIndex")
  specific_variable_name = statement.getProperty("SpecificVariableName")
  specific_variable_value = statement.getProperty("SpecificVariableValue")
  execution_mode = statement.getProperty("ExecutionMode")
  unwritable(loop_time, True)
  unwritable(infinite_loop, True)
  unwritable(reading_mode, True)
  unwritable(specific_indexes, True)
  unwritable(specific_variable_name, True)
  unwritable(specific_variable_value, True)
  unwritable(execution_mode, True)
  
  for property in statement.Properties:
    if "NextTransportPatternIn" in property.Name:
      unwritable(property, True)
      
def get_variable(variable_name, show_warning = True):   
  variable = routine.getProperty(variable_name)
  if not variable:
    if show_warning:
      print "No variable: " + variable_name + " found in routine: " + routine.Name
  return variable

def get_variable_value(variable, value):
  if variable.Type == VC_REAL:
    return float(value)
  elif variable.Type == VC_INTEGER:
    return int(float(value))
  elif variable.Type == VC_STRING:
    return value 
  elif variable.Type == VC_BOOLEAN:
    if value.lower() in ["1", "true", "yes", "t", "ok"]:
      return True
    else:
      return False  

def OnStart():
  global orders, pointer, current_loop, loaded

  pointer = 1
  current_loop = 1
  # Orders
  # 0: [variable_1, variable_2, ... ]
  # 1: [variable_1_value, variable_2_value, ... ]
  # 2: [variable_1_value, variable_2_value, ... ]
  orders = []
  loaded = False
  
  note.Note = note.Note.replace('\\r\\n','\\n')
  content = note.Note 
  # Into lines
  lines = [line.strip() for line in content.split('\\n') if line != "" and line[0] != '#']
  # Separate each line
  lines = [[variable.strip() for variable in line.split(',')] for line in lines]         
  # Check data
  for line_index, line in enumerate(lines):
    orders.append([])
    for variable_index, variable_name_or_value in enumerate(line):
      # Store variables
      if line_index == 0:      
        variable = get_variable(variable_name_or_value)
        if not variable:
          orders = []
          return          
        orders[-1].append(variable)
      # Store variables values
      else:
        variable = orders[0][variable_index]
        orders[-1].append(get_variable_value(variable, variable_name_or_value))

def search_variable_name_value(statement):
  note.Note = note.Note.replace('\\r\\n','\\n')
  content = note.Note   
  lines = [line.strip() for line in content.split('\\n') if line != "" and line[0] != '#']    
  line_indexes = [line_index for line_index, line in enumerate(lines) 
                  if (statement.SpecificVariableName in lines[0] and statement.SpecificVariableName != "" 
                  and statement.SpecificVariableValue in line and statement.SpecificVariableValue != "")]
  
  if len(line_indexes) == 0:
    print "Specific variable name and value is not found, set to index 1"
    return 1
  else:
    return line_indexes[0]

def search_signal_variable_name_value():
  note.Note = note.Note.replace('\\r\\n','\\n')
  content = note.Note   
  lines = [line.strip() for line in content.split('\\n') if line != "" and line[0] != '#']    
  line_indexes = [line_index for line_index, line in enumerate(lines) 
                  if (specific_variable_name_signal.Value in lines[0] and specific_variable_name_signal.Value != "" 
                  and specific_variable_value_signal.Value in line and specific_variable_value_signal.Value != "")]
  
  if len(line_indexes) == 0:
    print "Specific variable name and value is not found, set to index 1"
    return 1
  else:
    return line_indexes[0]

def set_next_statement_property(next_statement_property_value, xyz, variable_name, variable_type):  
  variable = get_variable(variable_name, False)
  if variable:
    if True: # variable.Type == variable_type:
      if (xyz == "X"):
        next_statement_property_value.X = variable.Value
      elif (xyz == "Y"):
        next_statement_property_value.Y = variable.Value
      elif (xyz == "Z"):
        next_statement_property_value.Z = variable.Value
      elif (xyz == "Tx"):
        next_statement_property_value.translateRel(-next_statement_property_value.P.X + variable.Value, 0.0, 0.0)        
      elif (xyz == "Ty"):
        next_statement_property_value.translateRel(0.0, -next_statement_property_value.P.Y + variable.Value, 0.0)        
      elif (xyz == "Tz"):
        next_statement_property_value.translateRel(0.0, 0.0, -next_statement_property_value.P.Z + variable.Value)  

  return next_statement_property_value
  
def update_next_statement(statement):
  # Check if next statement in transport pattern in
  current_index = routine.Statements.index(statement)
  if (current_index + 1) < len(routine.Statements):
    next_statement = routine.getStatement(current_index + 1)    
    if next_statement.Type == VC_STATEMENT_TRANSPORTPATTERNIN:
      
      filter = next_statement.setFilter(VC_FILTER_PRODUCTPROPERTY)
      filter.Name = "VariablesCache"
      filter.IsEnabled = False
      
      variable_name = statement.getProperty("NextTransportPatternIn::FilterProductPropertyName").Value      
      if variable_name != "":
        filter.IsEnabled = True
        filter.TargetProperty = variable_name
        variable_name = statement.getProperty("NextTransportPatternIn::FilterProductPropertyValue").Value      
        variable_value_string = str(get_variable(variable_name, False).Value)      
        filter.ComparisonValue = "\\"%s\\"" % variable_value_string
      
      # Offset
      pattern_offset = next_statement.Offset    
      # X
      variable_name = statement.getProperty("NextTransportPatternIn::Offset.Tx").Value     
      pattern_offset = set_next_statement_property(pattern_offset, "Tx", variable_name, VC_REAL)
      
      # Y
      variable_name = statement.getProperty("NextTransportPatternIn::Offset.Ty").Value     
      pattern_offset = set_next_statement_property(pattern_offset, "Ty", variable_name, VC_REAL)
      
      # Z
      variable_name = statement.getProperty("NextTransportPatternIn::Offset.Tz").Value     
      pattern_offset = set_next_statement_property(pattern_offset, "Tz", variable_name, VC_REAL)
      next_statement.Offset = pattern_offset
      
      # PatternCount
      pattern_count = next_statement.PatternCount      
      # X
      variable_name = statement.getProperty("NextTransportPatternIn::PatternCount.X").Value     
      pattern_count = set_next_statement_property(pattern_count, "X", variable_name, VC_INTEGER)
      
      # Y
      variable_name = statement.getProperty("NextTransportPatternIn::PatternCount.Y").Value     
      pattern_count = set_next_statement_property(pattern_count, "Y", variable_name, VC_INTEGER)
      
      # Z
      variable_name = statement.getProperty("NextTransportPatternIn::PatternCount.Z").Value     
      pattern_count = set_next_statement_property(pattern_count, "Z", variable_name, VC_INTEGER)
      next_statement.PatternCount = pattern_count

      # PatternStep
      # X
      pattern_step = next_statement.PatternStep
      variable_name = statement.getProperty("NextTransportPatternIn::PatternStep.X").Value     
      pattern_step = set_next_statement_property(pattern_step, "X", variable_name, VC_REAL)
      
      # Y
      variable_name = statement.getProperty("NextTransportPatternIn::PatternStep.Y").Value     
      pattern_step = set_next_statement_property(pattern_step, "Y", variable_name, VC_REAL)
      
      # Z
      variable_name = statement.getProperty("NextTransportPatternIn::PatternStep.Z").Value     
      pattern_step = set_next_statement_property(pattern_step, "Z", variable_name, VC_REAL)
      next_statement.PatternStep = pattern_step      

def OnStatementExecute(executor, statement):
  global orders, pointer, current_loop, loaded
    
  if orders == []:   
    print routine.Name + ": No orders"
    return  
  
  if statement.ExecutionMode == "Statement":

    if not statement.InfiniteLoop and current_loop > statement.LoopTime:
      delay(sys.float_info.max)
    
    # Update current progress
    statement.CurrentLoop = current_loop      
    statement.CurrentStep = "%i: %s" % (pointer, [orders[pointer]])  
    
    # Show data
    if not loaded:               
      statement.Variables = "%s" % [variable.Name for variable in orders[0]]
      step_values = ["%i: %s" % (order_index + 1, order) for order_index, order in enumerate(orders[1:])]
      statement.getProperty("Steps").StepValues = ["%i: %s" % (order_index + 1, order) for order_index, order in enumerate(orders[1:])]
      statement.getProperty("Steps").Value = "" if len(step_values) == 0 else step_values[0]

      if statement.ReadingMode == "Specific index":
        if statement.SpecificIndex == 0:
          statement.SpecificIndex = max(statement.SpecificIndex, 1)
          print "Specific index error: Starts from 1"
        pointer = statement.SpecificIndex        
      elif statement.ReadingMode == "Specific variable name and value":
        pointer = search_variable_name_value(statement)        
      
      loaded = True
    
    # Read data  
    for variable_index, variable in enumerate(orders[0]):
      variable.Value = orders[pointer][variable_index]
    
    # Update next statement - transport pattern in 
    if statement.getProperty("NextTransportPatternIn::Activate").Value:
      update_next_statement(statement)
      
    # Next    
    if statement.ReadingMode == "All lines":
      pointer += 1
      if pointer >= len(orders):     
        pointer = 1    
        current_loop += 1
    elif statement.ReadingMode in ["Specific index", "Specific variable name and value"]:
      current_loop += 1
      statement.CurrentLoop = current_loop      
      statement.CurrentStep = "%i: %s" % (pointer, [orders[pointer]])  

  elif statement.ExecutionMode == "Signal":
    
    # Update current progress
    statement.CurrentLoop = current_loop      
    statement.CurrentStep = "%i: %s" % (pointer, [orders[pointer]])  
    
    # Show data
    if not loaded:        
      statement.Variables = "%s" % [variable.Name for variable in orders[0]]
      step_values = ["%i: %s" % (order_index + 1, order) for order_index, order in enumerate(orders[1:])]
      statement.getProperty("Steps").StepValues = ["%i: %s" % (order_index + 1, order) for order_index, order in enumerate(orders[1:])]
      statement.getProperty("Steps").Value = "" if len(step_values) == 0 else step_values[0]
      loaded = True

    if statement.ReadingMode == "All lines":
      triggerCondition(lambda: getTrigger() == all_lines_signal)
    
    elif statement.ReadingMode == "Specific index":
      triggerCondition(lambda: getTrigger() == specific_index_signal)
      if specific_index_signal.Value == 0:        
        print "Specific index error: Starts from 1"      
      pointer = max(specific_index_signal.Value, 1)
    
    elif statement.ReadingMode == "Specific variable name and value":
      triggerCondition(lambda: getTrigger() == specific_variable_value_signal)      
      pointer = search_signal_variable_name_value()        

    # Read data  
    for variable_index, variable in enumerate(orders[0]):
      variable.Value = orders[pointer][variable_index]
    
    # Update next statement - transport pattern in 
    if statement.getProperty("NextTransportPatternIn::Activate").Value:
      update_next_statement(statement)
    
    # Next    
    if statement.ReadingMode == "All lines":
      pointer += 1
      if pointer >= len(orders):     
        pointer = 1    
        current_loop += 1
    elif statement.ReadingMode in ["Specific index", "Specific variable name and value"]:
      current_loop += 1
      statement.CurrentLoop = current_loop      
      statement.CurrentStep = "%i: %s" % (pointer, [orders[pointer]])  

'''

    statement = routine.addStatement("Process", 0, "Statement/ProcessModel/sGetAssemblyStepProperty")
    statement.Name = "VariablesCache"
    statement.Description = "VariablesCache"
    statement.getProperty('Name').IsVisible = False
    statement.getProperty('icon').IsVisible = False
    statement.getProperty('Tool').IsVisible = False
    statement.getProperty('Base').IsVisible = False
    statement.getProperty('Process').Value = handler
    
    p = statement.createProperty(VC_URI, 'Excel')  
    p.ChangeOnRebuild = False    
    p.IsVisible = True
    p = statement.createProperty(VC_STRING, 'SheetName', VC_PROPERTY_STEP)  
    p.StepValues = []
    p.ChangeOnRebuild = False    
    p.IsVisible = True
    p = statement.createProperty(VC_INTEGER, 'FromRow')  
    p.ChangeOnRebuild = False    
    p.IsVisible = True
    p = statement.createProperty(VC_INTEGER, 'ToRow')  
    p.ChangeOnRebuild = False    
    p.IsVisible = True
    p = statement.createProperty(VC_BUTTON, 'ReadExcel')  
    p.ChangeOnRebuild = False    
    p.IsVisible = True
    p = statement.createProperty(VC_INTEGER, 'CurrentLoop')  
    p.ChangeOnRebuild = False
    p.Value = 1
    p.IsVisible = True
    p = statement.createProperty(VC_STRING, 'CurrentStep')  
    p.ChangeOnRebuild = False
    p.Value = ""
    p.IsVisible = True
    p = statement.createProperty(VC_STRING, 'Variables')  
    p.ChangeOnRebuild = False
    p.Value = ""
    p.IsVisible = True
    p = statement.createProperty(VC_STRING, 'Steps', VC_PROPERTY_STEP)  
    p.ChangeOnRebuild = False
    p.Value = ""
    p.IsVisible = True    
    p = statement.createProperty(VC_INTEGER, 'LoopTime')  
    p.ChangeOnRebuild = False
    p.Value = 1
    p.IsVisible = True
    p = statement.createProperty(VC_BOOLEAN, 'InfiniteLoop')  
    p.ChangeOnRebuild = False
    p.Value = True
    p.IsVisible = True    
    p = statement.createProperty(VC_STRING, 'ReadingMode', VC_PROPERTY_STEP)  
    p.ChangeOnRebuild = False
    p.StepValues = ["All lines", "Specific index", "Specific variable name and value"]
    p.Value = "All lines"
    p.IsVisible = True 
    p = statement.createProperty(VC_INTEGER, 'SpecificIndex')
    p.ChangeOnRebuild = False  
    p.Value = 0
    p.IsVisible = True  
    p = statement.createProperty(VC_STRING, 'SpecificVariableName')
    p.ChangeOnRebuild = False  
    p.Value = ""
    p.IsVisible = True  
    p = statement.createProperty(VC_STRING, 'SpecificVariableValue')
    p.ChangeOnRebuild = False  
    p.Value = ""
    p.IsVisible = True  
    p = statement.createProperty(VC_STRING, 'ExecutionMode', VC_PROPERTY_STEP)  
    p.ChangeOnRebuild = False
    p.StepValues = ["Statement", "Signal"]
    p.Value = "Statement"
    p.IsVisible = True  
    p = statement.createProperty(VC_STRING, 'SpecificVariableValue')
    p.ChangeOnRebuild = False  
    p.Value = ""
    p.IsVisible = True 
    
    p = statement.createProperty(VC_BOOLEAN, 'NextTransportPatternIn::Activate')
    p.ChangeOnRebuild = False
    p.Value = False
    p.IsVisible = True 
    p = statement.createProperty(VC_BUTTON, 'NextTransportPatternIn::Update Variable Options')
    p.ChangeOnRebuild = False    
    p.IsVisible = True 
    p = statement.createProperty(VC_STRING, 'NextTransportPatternIn::Offset.Tx', VC_PROPERTY_STEP)
    p.ChangeOnRebuild = False
    p.IsVisible = True 
    p = statement.createProperty(VC_STRING, 'NextTransportPatternIn::Offset.Ty', VC_PROPERTY_STEP)
    p.ChangeOnRebuild = False  
    p.IsVisible = True 
    p = statement.createProperty(VC_STRING, 'NextTransportPatternIn::Offset.Tz', VC_PROPERTY_STEP)
    p.ChangeOnRebuild = False  
    p.IsVisible = True 
    p = statement.createProperty(VC_STRING, 'NextTransportPatternIn::Offset.Rx', VC_PROPERTY_STEP)
    p.ChangeOnRebuild = False
    p.IsVisible = True 
    p = statement.createProperty(VC_STRING, 'NextTransportPatternIn::Offset.Ry', VC_PROPERTY_STEP)
    p.ChangeOnRebuild = False  
    p.IsVisible = True 
    p = statement.createProperty(VC_STRING, 'NextTransportPatternIn::Offset.Rz', VC_PROPERTY_STEP)
    p.ChangeOnRebuild = False  
    p.IsVisible = True
    p = statement.createProperty(VC_STRING, 'NextTransportPatternIn::PatternCount.X', VC_PROPERTY_STEP)
    p.ChangeOnRebuild = False
    p.IsVisible = True 
    p = statement.createProperty(VC_STRING, 'NextTransportPatternIn::PatternCount.Y', VC_PROPERTY_STEP)
    p.ChangeOnRebuild = False  
    p.IsVisible = True 
    p = statement.createProperty(VC_STRING, 'NextTransportPatternIn::PatternCount.Z', VC_PROPERTY_STEP)
    p.ChangeOnRebuild = False  
    p.IsVisible = True 
    p = statement.createProperty(VC_STRING, 'NextTransportPatternIn::PatternStep.X', VC_PROPERTY_STEP)
    p.ChangeOnRebuild = False
    p.IsVisible = True 
    p = statement.createProperty(VC_STRING, 'NextTransportPatternIn::PatternStep.Y', VC_PROPERTY_STEP)
    p.ChangeOnRebuild = False  
    p.IsVisible = True 
    p = statement.createProperty(VC_STRING, 'NextTransportPatternIn::PatternStep.Z', VC_PROPERTY_STEP)
    p.ChangeOnRebuild = False  
    p.IsVisible = True 
    p = statement.createProperty(VC_STRING, 'NextTransportPatternIn::FilterProductPropertyName')
    p.ChangeOnRebuild = False  
    p.IsVisible = True 
    p = statement.createProperty(VC_STRING, 'NextTransportPatternIn::FilterProductPropertyValue', VC_PROPERTY_STEP)
    p.ChangeOnRebuild = False  
    p.IsVisible = True 
    
  else:
    print "Please select a statement"